package com.mt.wms.empty.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mt.wms.core.api.Assert;
import com.mt.wms.core.base.BaseService;
import com.mt.wms.core.dal.entity.*;
import com.mt.wms.core.dal.service.CurrTaskServiceBiz;
import com.mt.wms.core.dal.service.OrderInfoServiceBiz;
import com.mt.wms.core.params.BasePageParam;
import com.mt.wms.core.params.IdParam;
import com.mt.wms.core.vo.IdVo;
import com.mt.wms.core.vo.PageVo;
import com.mt.wms.core.vo.R;
import com.mt.wms.empty.controller.ApmsController;
import com.mt.wms.empty.enums.TaskTypeEnum;
import com.mt.wms.empty.params.CurrTaskQueryParam;
import com.mt.wms.empty.params.TaskCreateParam;
import com.mt.wms.empty.service.CurrTaskDetService;
import com.mt.wms.empty.service.CurrTaskService;
import com.mt.wms.empty.service.TaskDetHisService;
import com.mt.wms.empty.service.TaskHisService;
import com.mt.wms.empty.vo.ApmsEndProcessVo;
import com.mt.wms.empty.vo.CurrTaskMainQueryVo;
import com.mt.wms.empty.vo.CurrTaskQueryVo;
import com.mt.wms.empty.vo.CurrTaskVo;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * 服务实现类
 *
 * @author xcc
 * @date 2021年11月8日
 * @since 1.0
 */
@Service
@Transactional
public class CurrTaskServiceImpl extends BaseService implements CurrTaskService {

    @Autowired
    CurrTaskServiceBiz currTaskServiceBiz;

    @Autowired
    CurrTaskDetService currTaskDetService;

    @Autowired
    TaskHisService taskHisService;

    @Autowired
    TaskDetHisService taskDetHisService;

    @Autowired
    ApmsController apmsControl;

    @Autowired
    OrderInfoServiceBiz orderInfoServiceBiz;

    @Override
    public CurrTaskVo getCurrTask(IdParam idParam) {
        CurrTask byId = currTaskServiceBiz.getById(idParam.getId());
        CurrTaskVo build = CurrTaskVo.builder().build();
        BeanUtils.copyProperties(byId, build);
        return build;
    }

    @Override
    public R<IdVo> createProcessTask(TaskCreateParam param) {
        Assert.notNull("错误，加工类型不可为空！", param.getProcessType());
        CurrTask currTask = new CurrTask();
        BeanUtils.copyProperties(param, currTask);
        currTask.setTaskType(TaskTypeEnum.WTK.getValue());
        currTask.setTaskSource(0);
        setCommonField(currTask);
        currTaskServiceBiz.save(currTask);
        return successful(IdVo.builder().id(currTask.getId()).build());
    }

    @Override
    public R<PageVo<CurrTaskQueryVo>> currentTaskPage(CurrTaskQueryParam param) {
        QueryWrapper<CurrTask> currTaskQueryWrapper = new QueryWrapper<>();
        currTaskQueryWrapper.eq(StringUtils.isNotBlank(param.getVehicleId()), CurrTask.VEHICLE_ID, param.getVehicleId());
        currTaskQueryWrapper.eq(CurrTask.VALID, 1);
        Page<CurrTask> page = currTaskServiceBiz.page(new Page<>(param.getCurrent(), param.getSize()), currTaskQueryWrapper);
        return successful(new PageVo<>(page, CurrTaskQueryVo.class));
    }

    @Override
    public R<String> completeTask(IdParam param) {
        CurrTask byId = currTaskServiceBiz.getById(param.getId());
        TaskHis taskHis = new TaskHis();
        BeanUtils.copyProperties(byId, taskHis);
        taskHis.setStatus(2);
        taskHis.setId(null);
        //将主任务放入到历史表，清空id，防止重复
        R<IdVo> save = taskHisService.save(taskHis);
        byId.setValid(0);
        byId.setStatus(2);
        currTaskServiceBiz.updateById(byId);
        List<CurrTaskDet> taskDetailsByMainId = currTaskDetService.getTaskDetailsByMainId(IdParam.builder().id(byId.getId()).build());
        List<TaskDetHis> taskDetHis = com.mt.wms.core.utils.BeanUtils.copyList(taskDetailsByMainId, TaskDetHis.class);
        taskDetHis.forEach(detHis -> {
            detHis.setTaskId(save.getData().getId());
        });
        //将详细任务信息放入详细历史表
        taskDetHisService.saveList(taskDetHis);
        return successful("执行成功。");
    }

    @Override
    public R<String> deleteBySheetNo(String sheetNo) {
        CurrTask byId = currTaskServiceBiz.getById(new QueryWrapper<CurrTask>().eq(CurrTask.SHEET_NO, sheetNo));
        //如果尚未执行（状态 0）
        if (byId.getStatus() == 0)
        {

            R<ApmsEndProcessVo> apmsStartProcessR = apmsControl.deleteBySheetNo(sheetNo);
            if (apmsStartProcessR.getData().getSuccess())
            {
                currTaskServiceBiz.removeById(byId.getId());
                currTaskDetService.deleteByCurrTaskId(byId.getId());
                return successful("APMS生产单删除成功。");
            }
            else
            {
                return failed(apmsStartProcessR.getData().getMsg());
            }
        }
        return failed("无法删除，任务正在执行中。");

    }

    @Override
    public R<PageVo<CurrTaskMainQueryVo>> currentTaskMainPage(BasePageParam param) {
        QueryWrapper<CurrTask> currTaskQueryWrapper = new QueryWrapper<>();
        currTaskQueryWrapper.eq(CurrTask.STATUS, 1);
        currTaskQueryWrapper.eq(CurrTask.VALID, 1);
        Page<CurrTask> page = currTaskServiceBiz.page(new Page<>(param.getCurrent(), param.getSize()), currTaskQueryWrapper);
        PageVo<CurrTaskMainQueryVo> pageVo = new PageVo<>(page, CurrTaskMainQueryVo.class);
        Assert.notNull("无执行中的任务！", pageVo.getRecords());
        pageVo.getRecords().forEach(e ->
        {
            //根据当前主任务查详细任务
            List<CurrTaskDet> taskDetailsByMainId = currTaskDetService.getTaskDetailsByMainId(IdParam.builder().id(e.getId()).build());
            ArrayList<String> materialDes = new ArrayList<>();
            ArrayList<String> iden = new ArrayList<>();
            //写入材料名与标识卡号
            taskDetailsByMainId.forEach(a -> {
                materialDes.add(a.getMaterialDes());
                iden.add(a.getIdenCardNum());
            });
            e.setMaterials(materialDes);
            e.setIdens(iden);
            ArrayList<String> customers = new ArrayList<>();
            //根据标识卡号查订单的客户名并写入
            e.getIdens().forEach(id -> {
                        OrderInfo one = orderInfoServiceBiz.getOne(new QueryWrapper<OrderInfo>().eq(OrderInfo.IDEN_CARD_NUM, id));
                        customers.add(one.getCustomerName());
                    }
            );
            e.setCustomers(customers);
        });
        return successful(pageVo);
    }
}
